#ifdef DEBUG
#include <iostream>
#endif
#include <windows.h>
#include <string>

#include <SDL/SDL.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include "drawer.h"


Drawer::Drawer(int xres, int yres, int colors):
  _xres(xres),
  _yres(yres),
  _colors(colors),
  _screen(NULL),
  _xPOV(0.),
  _yPOV(0.),
  _zPOV(0.),
  _sensitivityX(400.),
  _sensitivityY(400.),
  _sensitivityZ(600.){
  
  SDL_Init(SDL_INIT_VIDEO);
  _screen = SDL_SetVideoMode(_xres,_yres,_colors,SDL_OPENGL|SDL_DOUBLEBUF);
  SDL_Flip(_screen);
  SDL_EnableKeyRepeat(10, 10);

}

void Drawer::setPOV(float x, float y, float z){
  _xPOV = x;
  _yPOV = y;
  _zPOV = z;
}

void Drawer::drawScene(){
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  /* init OpenGL */
  glEnable(GL_DEPTH_TEST);
  glShadeModel(GL_SMOOTH);
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45, (float) _xres / (float) _yres, 0.1, 50);
  /* Effective set of POV */
  gluLookAt(_xPOV, _yPOV, _zPOV, 0, 0, 1, 0,- 1, 0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  /* Draw the grid and enter in Flynn's world ! */
  glBegin(GL_LINES);{
    glColor3f(1.,1.,1.);
    /* horiz */
    for(float i = -3 ; i <= 3. ; i+=0.5){
      /* left */
      glVertex3d(-3.0,i, 0);
      glVertex3d(-3.0,i,10);
     
      /* right */
      glVertex3d(3.0,i, 0);
      glVertex3d(3.0,i,10);
      
      /* bottom */
      glVertex3d(i,-3.0, 0);
      glVertex3d(i,-3.0,10);
      
      /* top */
      glVertex3d(i, 3.,0.);
      glVertex3d(i,3.,10);
    }

    /* vertical */
    for(float i = 0 ; i <= 10. ; i+=0.5){
      /* left */
      glVertex3d(-3.0,-3., i);
      glVertex3d(-3.0,3., i);
     
      /* right */
      glVertex3d(3.0,-3., i);
      glVertex3d(3.0,3., i);

      /* bottom */
      glVertex3d(3.0,-3., i);
      glVertex3d(-3.0,-3., i);
      
      /* top */
      glVertex3d(3.0,3., i);
      glVertex3d(-3.0,3., i);
    }

  }glEnd();


  /* Draw a cube */
  glBegin(GL_QUADS);{
    glPushMatrix();{
      glRotatef(45,1,0,0);
      // front face
      glColor3f(1.,1.,0.);
      glVertex3d(0.,0.,0.);
      glVertex3d(0.,0.,1.);
      glVertex3d(1.,0.,1.);
      glVertex3d(1.,0.,0.);
      // back face
      glColor3f(1.,0.,1.);
      glVertex3d(0.,1.,0.);
      glVertex3d(0.,1.,1.);
      glVertex3d(1.,1.,1.);
      glVertex3d(1.,1.,0.);
      // bottom face
      glColor3f(0.,1.,1.);
      glVertex3d(0.,0.,0.);
      glVertex3d(0.,0.,1.);
      glVertex3d(0.,1.,1.);
      glVertex3d(0.,1.,0.);
      // top face
      glColor3f(1.,0.,0.);
      glVertex3d(1.,0.,0.);
      glVertex3d(1.,0.,1.);
      glVertex3d(1.,1.,1.);
      glVertex3d(1.,1.,0.);
      // left face
      glColor3f(0.,1.,0.);
      glVertex3d(0.,0.,0.);
      glVertex3d(1.,0.,0.);
      glVertex3d(1.,1.,0.);
      glVertex3d(0.,1.,0.);
      // right face
      glColor3f(0.,0.,1.);
      glVertex3d(0.,0.,1.);
      glVertex3d(1.,0.,1.);
      glVertex3d(1.,1.,1.);
      glVertex3d(0.,1.,1.);
    }glPopMatrix();
  }glEnd();

}

void Drawer::increaseX(){
  ++_sensitivityX;
}

void Drawer::decreaseX(){
  --_sensitivityX;
}

void Drawer::increaseY(){
  ++_sensitivityY;
}

void Drawer::decreaseY(){
  --_sensitivityY;
}

void Drawer::increaseZ(){
  ++_sensitivityZ;
}

void Drawer::decreaseZ(){
  --_sensitivityZ;
}

void Drawer::writeMsg(std::string msg){
  #ifdef DEBUG
  std::cout << msg << std::endl;
  #endif
    
}

void Drawer::headMoved(float x, float y, float z){
  #ifdef DEBUG
  std::cout << x << " " << y << " " << z << std::endl;
  #endif
  setPOV(x/_sensitivityX,-y/_sensitivityY,-z/_sensitivityZ);
}

Drawer::~Drawer(){
  SDL_FreeSurface(_screen);
}
