#ifndef DRAWER_H
#define DRAWER_H

#include <string>

#include <SDL/SDL.h>
#include "headlistener.h"

class Drawer : public HeadListener {
 public:
  /**
   * Open a window
   * @param xres xres of the screen in pixels
   * @param yres yres of the screen in pixels
   * @param color number of colors in bits
   */
  Drawer(int xres, int yres, int colors);
  
  /**
   * Set the point of view of the scene
   */
  void setPOV(float x, float y, float z);
  
  /**#include <string>
   * Draw the scene
   */
  void drawScene();
  
  /**
   * increase X sensitivity
   */
  void increaseX();
  
  /**
   * decrease X sensitivity
   */
  void decreaseX();
  
  /**
   * increase Y sensitivity
   */
  void increaseY();
 
  /**
   * decrease Y sensitivity
   */
  void decreaseY();
  
  /**
   * increase Z sensitivity
   */
  void increaseZ();
  
  /**
   * decrease Z sensitivity
   */
  void decreaseZ();
  
  /**
   * Implements HeadListener
   */
  void writeMsg(std::string msg);
  
  /**
   * Implements HeadListener
   */
  void headMoved(float x, float y, float z);

  ~Drawer();
 private:
  int _xres;
  int _yres;
  int _colors;
  SDL_Surface* _screen;
  
  float _xPOV;
  float _yPOV;
  float _zPOV;
  
  double _sensitivityX;
  double _sensitivityY;
  double _sensitivityZ;
};
#endif
